import { useQuery } from '@tanstack/react-query';

import { chatQueryKeys } from '../chat-query-keys';
import { chatApi, type PostGetChatWithReturn } from '../chat-api';
import NetInfo from '@react-native-community/netinfo';

import type { BaseAxiosError } from '../../../../types';
import { enforceStorageLimit, saveMessagesToStorage, storage, StoreType } from 'src/storage';

export const usePostGetChatWithQuery = (
  token: string,
  uid: number,
  no_of_messages: number,
  previous_than_message_id: number,
  enabled: boolean
) => {
  return useQuery<PostGetChatWithReturn, BaseAxiosError>({
    queryKey: chatQueryKeys.getChatWith(token, uid, no_of_messages, previous_than_message_id),
    queryFn: async () => {
      const response = await chatApi.getChatWith(
        token,
        uid,
        no_of_messages,
        previous_than_message_id
      );

      const netInfoState = await NetInfo.fetch();

      if (!response.data.messages.length && previous_than_message_id === -1) {
        storage.remove(`chat_${uid}`);
        storage.remove(`chat_${uid}_updatedAt`);
      } else if (previous_than_message_id === -1 && netInfoState.isConnected) {
        saveMessagesToStorage(uid, response.data.messages);
      }

      enforceStorageLimit();

      return response.data;
    },
    enabled,
    initialData: () => {
      const storedMessages = storage.get(`chat_${uid}`, StoreType.STRING) as string;
      let messages = storedMessages ? JSON.parse(storedMessages) : [];

      return messages.length ? ({ messages, result: 'OK' } as PostGetChatWithReturn) : undefined;
    },
    initialDataUpdatedAt:
      (storage.get(`chat_${uid}_updatedAt`, StoreType.NUMBER) as number) || undefined
  });
};
