import { useQuery } from '@tanstack/react-query';

import { chatQueryKeys } from '../chat-query-keys';
import { chatApi, type PostGetGroupChatWithReturn } from '../chat-api';

import type { BaseAxiosError } from '../../../../types';
import { enforceStorageLimit, saveMessagesToStorage, storage, StoreType } from 'src/storage';

export const usePostGetGroupChatQuery = (
  token: string,
  group_token: string,
  no_of_messages: number,
  previous_than_message_id: number,
  enabled: boolean
) => {
  return useQuery<PostGetGroupChatWithReturn, BaseAxiosError>({
    queryKey: chatQueryKeys.getGroupChat(
      token,
      group_token,
      no_of_messages,
      previous_than_message_id
    ),
    queryFn: async () => {
      const response = await chatApi.getGroupChat(
        token,
        group_token,
        no_of_messages,
        previous_than_message_id
      );

      if (!response.data.messages.length && previous_than_message_id === -1) {
        storage.remove(`chat_${group_token}`);
        storage.remove(`chat_${group_token}_updatedAt`);
      } else if (previous_than_message_id === -1) {
        saveMessagesToStorage(group_token, response.data);
      }

      enforceStorageLimit();
      return response.data;
    },
    enabled,
    initialData: () => {
      const storedMessages = storage.get(`chat_${group_token}`, StoreType.STRING) as string;
      
      return storedMessages
        ? (JSON.parse(storedMessages) as PostGetGroupChatWithReturn)
        : undefined;
    },
    initialDataUpdatedAt:
      (storage.get(`chat_${group_token}_updatedAt`, StoreType.NUMBER) as number) || undefined
  });
};
